package cellar

import (
	"fmt"
	"math/rand"
	"os/exec"
	"strings"
	"time"

	"github.com/marc-gr/cellar/config"
	irc "github.com/thoj/go-ircevent"
)

func nickName() string {
	rand.Seed(time.Now().UnixNano())
	return fmt.Sprintf("cellar%09d", rand.Int63n(999999999))
}

func (c *Cellar) discoverLoop() error {
	nickName := nickName()
	ircobj := irc.IRC(nickName, nickName)
	ircobj.Debug = false
	if err := ircobj.Connect("chat.freenode.net:6667"); err != nil {
		return err
	}
	go ircobj.Loop()

	ircobj.AddCallback("PRIVMSG", func(event *irc.Event) {
		cmd := exec.Command(
			"ipfs",
			"-c", config.IPFSPath(),
			"swarm",
			"connect",
			event.Message(),
		)
		fmt.Println("connecting to", event.Message())
		out, err := cmd.CombinedOutput()
		if err != nil {
			fmt.Println(err)
		}
		fmt.Println(string(out))
	})

	ircobj.AddCallback("353", func(event *irc.Event) {
		if len(event.Arguments) < 4 {
			return
		}
		for _, nick := range strings.Split(event.Arguments[3], " ") {
			nick = strings.Trim(nick, "@")
			if nick == nickName || !strings.HasPrefix(nick, "cellar") {
				continue
			}
			ircobj.Privmsg(nick, c.ipfsNodeAddr)
		}
	})

	_, w := time.Now().ISOWeek()

	ircobj.Join(fmt.Sprintf("#cellar-%d", w))

	getList := func() {
		if !ircobj.Connected() {
			_ = ircobj.Reconnect()
		}

		ircobj.SendRawf("NAMES #cellar-%d\r\n", w)
	}

	go func() {
		// TODO: change to minutes
		ticker := time.NewTicker(5 * time.Second)
		defer ircobj.Disconnect()
		for range ticker.C {
			getList()
		}
	}()

	return nil
}
