#!/bin/bash

set -e

echo
echo "Starting up cluster with 4 nodes."
echo "It is using IRC for peer discovery."
echo "It is also set up as a private IPFS/IPFSCluster network."
sleep 1
echo
docker-compose build node1 node-test1 >/dev/null
docker-compose up -d --force-recreate node-test1 node1 node2 node3
echo
echo "Waiting for it to be up..."
sleep 10

echo "Creating some files on node-test1"
sleep 1
echo
docker-compose exec node-test1 bash -c "./create-files.sh"
PINS=$(docker-compose exec node-test1 bash -c './getpins.sh')
echo
echo "Wait for syncing to trigger..."
sleep 30
echo "Validate if node-test1 has files..."
docker-compose exec node-test1 bash -c "./validate-files-unencrypted.sh"
echo

echo "Check if they were replicated in other nodes."
echo "Other nodes will only have access to the encrypted files."
sleep 1
echo
echo "node1:"
docker-compose exec node1 bash -c "./validate-files-encrypted.sh $PINS"
echo "node2:"
docker-compose exec node2 bash -c "./validate-files-encrypted.sh $PINS"
echo "node3:"
docker-compose exec node3 bash -c "./validate-files-encrypted.sh $PINS"
echo

echo "Shutting down node-test1 and creating another one with the same identity."
echo "This way we can validate if we can access the file contents."
sleep 1
echo
docker-compose stop node-test1
docker-compose up -d --force-recreate node-test2
echo
echo "Waiting for it to be up and in sync..."
sleep 30
echo "Validate if node-test2 has files..."
docker-compose exec node-test2 bash -c "./validate-files-unencrypted.sh"
docker-compose down